@@ -52,7 +52,8 @@ class ApiController(base.BaseController):
 
         self._identify_user()
         try:
-            context = {'model': model, 'user': c.user or c.author}
+            context = {'model': model, 'user': c.user or c.author,
+                       'auth_user_obj': c.userobj}
             logic.check_access('site_read', context)
         except NotAuthorized:
             response_msg = self._finish(403,
@@ -113,8 +114,10 @@ class ApiController(base.BaseController):
 
         return self._finish(status_int, response_data, content_type)
 
-    def _finish_not_authz(self):
+    def _finish_not_authz(self, extra_msg=None):
         response_data = _('Access denied')
+        if extra_msg:
+            response_data = '%s - %s' % (response_data, extra_msg)
         return self._finish(403, response_data, 'json')
 
     def _finish_not_found(self, extra_msg=None):
@@ -161,7 +164,7 @@ class ApiController(base.BaseController):
                 _('Action name not known: %s') % logic_function)
 
         context = {'model': model, 'session': model.Session, 'user': c.user,
-                   'api_version': ver}
+                   'api_version': ver, 'auth_user_obj': c.userobj}
         model.Session()._context = context
         return_dict = {'help': function.__doc__}
         try:
@@ -193,10 +196,14 @@ class ApiController(base.BaseController):
                                     'data': request_data}
             return_dict['success'] = False
             return self._finish(400, return_dict, content_type='json')
-        except NotAuthorized:
+        except NotAuthorized, e:
             return_dict['error'] = {'__type': 'Authorization Error',
                                     'message': _('Access denied')}
             return_dict['success'] = False
+            
+            if e.extra_msg:
+                return_dict['error']['message'] += ': %s' % e.extra_msg
+
             return self._finish(403, return_dict, content_type='json')
         except NotFound, e:
             return_dict['error'] = {'__type': 'Not Found Error',
@@ -248,7 +255,8 @@ class ApiController(base.BaseController):
 
     def list(self, ver=None, register=None, subregister=None, id=None):
         context = {'model': model, 'session': model.Session,
-                   'user': c.user, 'api_version': ver}
+                   'user': c.user, 'api_version': ver,
+                   'auth_user_obj': c.userobj}
         log.debug('listing: %s' % context)
         action_map = {
             'revision': 'revision_list',
@@ -275,8 +283,9 @@ class ApiController(base.BaseController):
         except NotFound, e:
             extra_msg = e.extra_msg
             return self._finish_not_found(extra_msg)
-        except NotAuthorized:
-            return self._finish_not_authz()
+        except NotAuthorized, e:
+            extra_msg = e.extra_msg
+            return self._finish_not_authz(extra_msg)
 
     def show(self, ver=None, register=None, subregister=None,
              id=None, id2=None):
@@ -292,7 +301,7 @@ class ApiController(base.BaseController):
             action_map[('dataset', type)] = 'package_relationships_list'
 
         context = {'model': model, 'session': model.Session, 'user': c.user,
-                   'api_version': ver}
+                   'api_version': ver, 'auth_user_obj': c.userobj}
         data_dict = {'id': id, 'id2': id2, 'rel': subregister}
 
         log.debug('show: %s' % context)
@@ -306,8 +315,9 @@ class ApiController(base.BaseController):
         except NotFound, e:
             extra_msg = e.extra_msg
             return self._finish_not_found(extra_msg)
-        except NotAuthorized:
-            return self._finish_not_authz()
+        except NotAuthorized, e:
+            extra_msg = e.extra_msg
+            return self._finish_not_authz(extra_msg)
 
     def _represent_package(self, package):
         return package.as_dict(ref_package_by=self.ref_package_by,
@@ -327,7 +337,7 @@ class ApiController(base.BaseController):
             action_map[('dataset', type)] = 'package_relationship_create_rest'
 
         context = {'model': model, 'session': model.Session, 'user': c.user,
-                   'api_version': ver}
+                   'api_version': ver, 'auth_user_obj': c.userobj}
         log.debug('create: %s' % (context))
         try:
             request_data = self._get_request_data()
@@ -352,8 +362,9 @@ class ApiController(base.BaseController):
                                           data_dict.get("id")))
             return self._finish_ok(response_data,
                                    resource_location=location)
-        except NotAuthorized:
-            return self._finish_not_authz()
+        except NotAuthorized, e:
+            extra_msg = e.extra_msg
+            return self._finish_not_authz(extra_msg)
         except NotFound, e:
             extra_msg = e.extra_msg
             return self._finish_not_found(extra_msg)
@@ -390,7 +401,7 @@ class ApiController(base.BaseController):
             action_map[('dataset', type)] = 'package_relationship_update_rest'
 
         context = {'model': model, 'session': model.Session, 'user': c.user,
-                   'api_version': ver, 'id': id}
+                   'api_version': ver, 'id': id, 'auth_user_obj': c.userobj}
         log.debug('update: %s' % (context))
         try:
             request_data = self._get_request_data()
@@ -408,8 +419,9 @@ class ApiController(base.BaseController):
         try:
             response_data = action(context, data_dict)
             return self._finish_ok(response_data)
-        except NotAuthorized:
-            return self._finish_not_authz()
+        except NotAuthorized, e:
+            extra_msg = e.extra_msg
+            return self._finish_not_authz(extra_msg)
         except NotFound, e:
             extra_msg = e.extra_msg
             return self._finish_not_found(extra_msg)
@@ -442,7 +454,7 @@ class ApiController(base.BaseController):
             action_map[('dataset', type)] = 'package_relationship_delete_rest'
 
         context = {'model': model, 'session': model.Session, 'user': c.user,
-                   'api_version': ver}
+                   'api_version': ver, 'auth_user_obj': c.userobj}
 
         data_dict = {'id': id, 'id2': id2, 'rel': subregister}
 
@@ -456,8 +468,9 @@ class ApiController(base.BaseController):
         try:
             response_data = action(context, data_dict)
             return self._finish_ok(response_data)
-        except NotAuthorized:
-            return self._finish_not_authz()
+        except NotAuthorized, e:
+            extra_msg = e.extra_msg
+            return self._finish_not_authz(extra_msg)
         except NotFound, e:
             extra_msg = e.extra_msg
             return self._finish_not_found(extra_msg)
@@ -597,7 +610,7 @@ class ApiController(base.BaseController):
         c.q = request.params.get('q', '')
 
         context = {'model': model, 'session': model.Session,
-                   'user': c.user or c.author}
+                   'user': c.user or c.author, 'auth_user_obj': c.userobj}
 
         tag_names = get_action('tag_list')(context, {})
         results = []
@@ -632,7 +645,7 @@ class ApiController(base.BaseController):
         user_list = []
         if q:
             context = {'model': model, 'session': model.Session,
-                       'user': c.user or c.author}
+                       'user': c.user or c.author, 'auth_user_obj': c.userobj}
 
             data_dict = {'q': q, 'limit': limit}
 
@@ -696,7 +709,7 @@ class ApiController(base.BaseController):
         package_dicts = []
         if q:
             context = {'model': model, 'session': model.Session,
-                       'user': c.user or c.author}
+                       'user': c.user or c.author, 'auth_user_obj': c.userobj}
 
             data_dict = {'q': q, 'limit': limit}
 
@@ -712,7 +725,7 @@ class ApiController(base.BaseController):
         tag_names = []
         if q:
             context = {'model': model, 'session': model.Session,
-                       'user': c.user or c.author}
+                       'user': c.user or c.author, 'auth_user_obj': c.userobj}
 
             data_dict = {'q': q, 'limit': limit}
 
@@ -731,7 +744,7 @@ class ApiController(base.BaseController):
         formats = []
         if q:
             context = {'model': model, 'session': model.Session,
-                       'user': c.user or c.author}
+                       'user': c.user or c.author, 'auth_user_obj': c.userobj}
             data_dict = {'q': q, 'limit': limit}
             formats = get_action('format_autocomplete')(context, data_dict)
 
@@ -826,7 +839,9 @@ class ApiController(base.BaseController):
         cls.log.debug('Retrieving request POST: %r' % request.POST)
         cls.log.debug('Retrieving request GET: %r' % request.GET)
         request_data = None
-        if request.POST:
+        if request.POST and request.content_type == 'multipart/form-data':
+            request_data = dict(request.POST)
+        elif request.POST:
             try:
                 keys = request.POST.keys()
                 # Parsing breaks if there is a = in the value, so for now
@@ -860,7 +875,7 @@ class ApiController(base.BaseController):
                     raise ValueError(msg)
                 else:
                     request_data = {}
-        if request_data:
+        if request_data and request.content_type != 'multipart/form-data':
             try:
                 request_data = h.json.loads(request_data, encoding='utf8')
             except ValueError, e:
