@@ -1,6 +1,8 @@
 import re
+import os
 import logging
 import genshi
+import cgi
 import datetime
 from urllib import urlencode
 
@@ -166,6 +168,9 @@ class GroupController(base.BaseController):
 
     def read(self, id, limit=20):
         group_type = self._get_group_type(id.split('@')[0])
+        if group_type != self.group_type:
+            abort(404, _('Incorrect group type'))
+
         context = {'model': model, 'session': model.Session,
                    'user': c.user or c.author,
                    'schema': self._db_to_form_schema(group_type=group_type),
@@ -176,6 +181,9 @@ class GroupController(base.BaseController):
         q = c.q = request.params.get('q', '')
 
         try:
+            # Do not query for the group datasets when dictizing, as they will
+            # be ignored and get requested on the controller anyway
+            context['include_datasets'] = False
             c.group_dict = self._action('group_show')(context, data_dict)
             c.group = context['group']
         except NotFound:
@@ -278,10 +286,11 @@ class GroupController(base.BaseController):
 
             facets = OrderedDict()
 
-            default_facet_titles = {'groups': _('Groups'),
+            default_facet_titles = {'organization': _('Organizations'),
+                                    'groups': _('Groups'),
                                     'tags': _('Tags'),
                                     'res_format': _('Formats'),
-                                    'license_id': _('License')}
+                                    'license_id': _('Licenses')}
 
             for facet in g.facets:
                 if facet in default_facet_titles:
@@ -324,6 +333,7 @@ class GroupController(base.BaseController):
                 items_per_page=limit
             )
 
+            c.group_dict['package_count'] = query['count']
             c.facets = query['facets']
             maintain.deprecate_context_item('facets',
                                             'Use `c.search_facets` instead.')
@@ -344,6 +354,9 @@ class GroupController(base.BaseController):
             c.facets = {}
             c.page = h.Page(collection=[])
 
+        self._setup_template_variables(context, {'id':id},
+            group_type=group_type)
+
     def bulk_process(self, id):
         ''' Allow bulk processing of datasets for an organization.  Make
         private/public or delete. For organization admins.'''
@@ -363,6 +376,9 @@ class GroupController(base.BaseController):
         data_dict = {'id': id}
 
         try:
+            # Do not query for the group datasets when dictizing, as they will
+            # be ignored and get requested on the controller anyway
+            context['include_datasets'] = False
             c.group_dict = self._action('group_show')(context, data_dict)
             c.group = context['group']
         except NotFound:
@@ -370,16 +386,28 @@ class GroupController(base.BaseController):
         except NotAuthorized:
             abort(401, _('Unauthorized to read group %s') % id)
 
-        # Search within group
-        action = request.params.get('bulk_action')
+        #use different form names so that ie7 can be detected
+        form_names = set(["bulk_action.public", "bulk_action.delete",
+                          "bulk_action.private"])
+        actions_in_form = set(request.params.keys())
+        actions = form_names.intersection(actions_in_form)
         # If no action then just show the datasets
-        if not action:
+        if not actions:
             # unicode format (decoded from utf8)
             limit = 500
             self._read(id, limit)
             c.packages = c.page.items
             return render(self._bulk_process_template(group_type))
 
+        #ie7 puts all buttons in form params but puts submitted one twice
+        for key, value in dict(request.params.dict_of_lists()).items():
+            if len(value) == 2:
+                action = key.split('.')[-1]
+                break
+        else:
+            #normal good browser form submission
+            action = actions.pop().split('.')[-1]
+
         # process the action first find the datasets to perform the action on.
         # they are prefixed by dataset_ in the form data
         datasets = []
@@ -421,6 +449,9 @@ class GroupController(base.BaseController):
             return self._save_new(context, group_type)
 
         data = data or {}
+        if not data.get('image_url', '').startswith('http'):
+            data.pop('image_url', None)
+
         errors = errors or {}
         error_summary = error_summary or {}
         vars = {'data': data, 'errors': errors,
@@ -520,7 +551,6 @@ class GroupController(base.BaseController):
             data_dict['id'] = id
             context['allow_partial_update'] = True
             group = self._action('group_update')(context, data_dict)
-
             if id != group['name']:
                 self._force_reindex(group)
 
@@ -612,6 +642,19 @@ class GroupController(base.BaseController):
                 data_dict = clean_dict(dict_fns.unflatten(
                     tuplize_dict(parse_params(request.params))))
                 data_dict['id'] = id
+
+                email = data_dict.get('email')
+                if email:
+                    user_data_dict = {
+                        'email': email,
+                        'group_id': data_dict['id'],
+                        'role': data_dict['role']
+                    }
+                    del data_dict['email']
+                    user_dict = self._action('user_invite')(context,
+                            user_data_dict)
+                    data_dict['username'] = user_dict['name']
+
                 c.group_dict = self._action('group_member_create')(context, data_dict)
                 self._redirect_to(controller='group', action='members', id=id)
             else:
@@ -622,7 +665,10 @@ class GroupController(base.BaseController):
                 else:
                     c.user_role = 'member'
                 c.group_dict = self._action('group_show')(context, {'id': id})
-                c.roles = self._action('member_roles_list')(context, {})
+                group_type = 'organization' if c.group_dict['is_organization'] else 'group'
+                c.roles = self._action('member_roles_list')(
+                    context, {'group_type': group_type}
+                )
         except NotAuthorized:
             abort(401, _('Unauthorized to add member to group %s') % '')
         except NotFound:
@@ -739,11 +785,8 @@ class GroupController(base.BaseController):
 
         context = {'model': model, 'session': model.Session,
                    'user': c.user or c.author, 'for_view': True}
-        data_dict = {'id': id}
-
         try:
-            c.group_dict = get_action('group_show')(context, data_dict)
-            c.group = context['group']
+            c.group_dict = self._get_group_dict(id)
         except NotFound:
             abort(404, _('Group not found'))
         except NotAuthorized:
@@ -753,10 +796,9 @@ class GroupController(base.BaseController):
 
         # Add the group's activity stream (already rendered to HTML) to the
         # template context for the group/read.html template to retrieve later.
-        c.group_activity_stream = get_action('group_activity_list_html')(
+        c.group_activity_stream = self._action('group_activity_list_html')(
             context, {'id': c.group_dict['id'], 'offset': offset})
 
-        #return render('group/activity_stream.html')
         return render(self._activity_template(c.group_dict['type']))
 
     def follow(self, id):
@@ -814,8 +856,13 @@ class GroupController(base.BaseController):
         return render(self._admins_template(c.group_dict['type']))
 
     def about(self, id):
+        context = {'model': model, 'session': model.Session,
+                   'user': c.user or c.author}
         c.group_dict = self._get_group_dict(id)
-        return render(self._about_template(c.group_dict['type']))
+        group_type = c.group_dict['type']
+        self._setup_template_variables(context, {'id': id},
+                                       group_type=group_type)
+        return render(self._about_template(group_type))
 
     def _get_group_dict(self, id):
         ''' returns the result of group_show action or aborts if there is a
@@ -824,7 +871,7 @@ class GroupController(base.BaseController):
                    'user': c.user or c.author,
                    'for_view': True}
         try:
-            return get_action('group_show')(context, {'id': id})
+            return self._action('group_show')(context, {'id': id})
         except NotFound:
             abort(404, _('Group not found'))
         except NotAuthorized:
