@@ -6,7 +6,8 @@ import warnings
 with warnings.catch_warnings():
     warnings.filterwarnings('ignore', '.*compile_mappers.*')
     import formalchemy
-from ckan.lib.helpers import OrderedDict
+
+from ckan.common import OrderedDict
 
 months = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December']
 
