@@ -1,3 +1,5 @@
+import json
+
 import ckan.model as model
 import ckan.lib.navl.dictization_functions as df
 import ckan.lib.field_types as field_types
@@ -169,3 +171,13 @@ def convert_group_name_or_id_to_id(group_name_or_id, context):
     if not result:
         raise df.Invalid('%s: %s' % (_('Not found'), _('Group')))
     return result.id
+
+
+def convert_to_json_if_string(value, context):
+    if isinstance(value, basestring):
+        try:
+            return json.loads(value)
+        except ValueError:
+            raise df.Invalid(_('Could not parse as valid JSON'))
+    else:
+        return value
