@@ -36,10 +36,22 @@
     null,
     "Skrýt"
   ],
+  "Image": [
+    null,
+    "Obrázek"
+  ],
   "Input is too short, must be at least one character": [
     null,
     "Zadaný vstup je příliš krátký, musíte zadat alespoň jeden znak"
   ],
+  "Link": [
+    null,
+    "Odkaz"
+  ],
+  "Link to a URL on the internet (you can also link to an API)": [
+    null,
+    "Odkaz na internetovou URL adresu (můžete také zadat odkaz na API)"
+  ],
   "Loading...": [
     null,
     "Nahrávám ..."
@@ -52,10 +64,30 @@
     null,
     "Prosím potvrďte akci"
   ],
+  "Remove": [
+    null,
+    "Odstranit"
+  ],
+  "Reorder resources": [
+    null,
+    "Změnit pořadí zdrojů"
+  ],
+  "Reset this": [
+    null,
+    "Resetovat"
+  ],
   "Resource uploaded": [
     null,
     "Zdroj nahrán"
   ],
+  "Save order": [
+    null,
+    "Uložit pořadí"
+  ],
+  "Saving...": [
+    null,
+    "Ukládám..."
+  ],
   "Show more": [
     null,
     "Ukázat více"
@@ -92,10 +124,18 @@
     null,
     "Přestat sledovat"
   ],
+  "Upload": [
+    null,
+    "Nahrát"
+  ],
   "Upload a file": [
     null,
     "Nahrát soubor"
   ],
+  "Upload a file on your computer": [
+    null,
+    "Nahrát soubor na Váš počítač"
+  ],
   "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
     null,
     "Právě nahráváte soubor. Jste si opravdu jistí, že chcete tuto stránku opustit a ukončit tak nahrávání?"
