@@ -36,10 +36,22 @@
     null,
     "Esconder"
   ],
+  "Image": [
+    null,
+    "Imagen"
+  ],
   "Input is too short, must be at least one character": [
     null,
     "Ingreso es muy corto, debe tener al menos un caracter"
   ],
+  "Link": [
+    null,
+    "Enlace"
+  ],
+  "Link to a URL on the internet (you can also link to an API)": [
+    null,
+    "Enlazar a una URL de internet (también puede enlazar a una API)"
+  ],
   "Loading...": [
     null,
     "Cargando ..."
@@ -52,10 +64,30 @@
     null,
     "Por favor confirme la acción"
   ],
+  "Remove": [
+    null,
+    "Quitar"
+  ],
+  "Reorder resources": [
+    null,
+    "Reordenar recursos"
+  ],
+  "Reset this": [
+    null,
+    "Reiniciar esto"
+  ],
   "Resource uploaded": [
     null,
     "Recurso subido"
   ],
+  "Save order": [
+    null,
+    "Guardar orden"
+  ],
+  "Saving...": [
+    null,
+    "Guardando..."
+  ],
   "Show more": [
     null,
     "Mostrar más"
@@ -92,10 +124,18 @@
     null,
     "Dejar de seguir"
   ],
+  "Upload": [
+    null,
+    "Subir"
+  ],
   "Upload a file": [
     null,
     "Sube un archivo"
   ],
+  "Upload a file on your computer": [
+    null,
+    "Subir un fichero a su PC"
+  ],
   "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
     null,
     "Estás subiendo un fichero. Estás seguro que quieres salir y parar esta subida?"
