@@ -3,5 +3,9 @@
     "domain": "ckan",
     "lang": "km",
     "plural-forms": "nplurals=1; plural=0;"
-  }
+  },
+  "Image": [
+    null,
+    "រូបភាព"
+  ]
 }
\ No newline at end of file
