@@ -36,6 +36,10 @@
     null,
     "Спрятать"
   ],
+  "Image": [
+    null,
+    "Картинка"
+  ],
   "Input is too short, must be at least one character": [
     null,
     "Слишком коротко, должен быть хотя бы один символ"
@@ -52,6 +56,10 @@
     null,
     "Пожалуйста, подтвердите действие"
   ],
+  "Remove": [
+    null,
+    "Удалить"
+  ],
   "Resource uploaded": [
     null,
     "Ресурс загружен"
@@ -92,6 +100,10 @@
     null,
     "Не следовать"
   ],
+  "Upload": [
+    null,
+    "Загрузка"
+  ],
   "Upload a file": [
     null,
     "Вложить файл"
