@@ -1,91 +1,22 @@
-import cgi
 import datetime
 
-from paste.fixture import AppError
-from pylons import config
-from pylons import c
+from pylons import config, c
 from genshi.core import escape as genshi_escape
 from difflib import unified_diff
-from nose.plugins.skip import SkipTest
 from nose.tools import assert_equal
 
 from ckan.tests import *
+import ckan.tests as tests
 from ckan.tests.html_check import HtmlCheckMethods
 from ckan.tests.pylons_controller import PylonsTestCase
 from base import FunctionalTestCase
 import ckan.model as model
 from ckan.lib.create_test_data import CreateTestData
-import ckan.lib.helpers as h
-import ckan.lib.search as search
 from ckan.logic.action import get, update
-from ckan.controllers.package import PackageController
-from ckan.plugins import SingletonPlugin, implements, IPackageController
 from ckan import plugins
-from ckan.rating import set_rating
 from ckan.lib.search.common import SolrSettings
 
 
-class MockPackageControllerPlugin(SingletonPlugin):
-    implements(IPackageController)
-
-    def __init__(self):
-        from collections import defaultdict
-        self.calls = defaultdict(int)
-
-    def read(self, entity):
-        self.calls['read'] += 1
-
-    def create(self, entity):
-        self.calls['create'] += 1
-
-    def edit(self, entity):
-        self.calls['edit'] += 1
-
-    def authz_add_role(self, object_role):
-        self.calls['authz_add_role'] += 1
-
-    def authz_remove_role(self, object_role):
-        self.calls['authz_remove_role'] += 1
-
-    def delete(self, entity):
-        self.calls['delete'] += 1
-
-    def before_search(self, search_params):
-        self.calls['before_search'] += 1
-        return search_params
-
-    def after_search(self, search_results, search_params):
-        self.calls['after_search'] += 1
-        return search_results
-
-    def before_index(self, data_dict):
-        self.calls['before_index'] += 1
-        return data_dict
-
-    def before_view(self, data_dict):
-        self.calls['before_view'] += 1
-        return data_dict
-
-    def after_create(self, context, data_dict):
-        self.calls['after_create'] += 1
-        self.id_in_dict = 'id' in data_dict
-
-        return data_dict
-
-    def after_update(self, context, data_dict):
-        self.calls['after_update'] += 1
-        return data_dict
-
-    def after_delete(self, context, data_dict):
-        self.calls['after_delete'] += 1
-        return data_dict
-
-    def after_show(self, context, data_dict):
-        self.calls['after_show'] += 1
-        return data_dict
-
-    def update_facet_titles(self, facet_titles):
-        return facet_titles
 
 
 existing_extra_html = ('<label class="field_opt" for="Package-%(package_id)s-extras-%(key)s">%(capitalized_key)s</label>', '<input id="Package-%(package_id)s-extras-%(key)s" name="Package-%(package_id)s-extras-%(key)s" size="20" type="text" value="%(value)s">')
@@ -382,15 +313,15 @@ class TestReadOnly(TestPackageForm, HtmlCheckMethods, PylonsTestCase):
         assert name in res
 
     def test_read_plugin_hook(self):
-        plugin = MockPackageControllerPlugin()
-        plugins.load(plugin)
+        plugins.load('test_package_controller_plugin')
+        plugin = plugins.get_plugin('test_package_controller_plugin')
         name = u'annakarenina'
         offset = url_for(controller='package', action='read', id=name)
         res = self.app.get(offset)
 
         assert plugin.calls['read'] == 1, plugin.calls
         assert plugin.calls['after_show'] == 1, plugin.calls
-        plugins.unload(plugin)
+        plugins.unload('test_package_controller_plugin')
 
     def test_resource_list(self):
         # TODO restore this test. It doesn't make much sense with the
@@ -530,7 +461,7 @@ class TestReadAtRevision(FunctionalTestCase, HtmlCheckMethods):
         side_html = self.named_div('sidebar', res)
         print 'MAIN', main_html
         assert 'This is an old revision of this dataset' in main_html
-        assert 'at Jan 01, 2011, 00:00' in main_html
+        assert 'at January 1, 2011, 00:00' in main_html
         self.check_named_element(main_html, 'a', 'href="/dataset/%s"' % self.pkg_name)
         print 'PKG', pkg_html
         assert 'title1' in res
@@ -548,7 +479,7 @@ class TestReadAtRevision(FunctionalTestCase, HtmlCheckMethods):
         side_html = self.named_div('sidebar', res)
         print 'MAIN', main_html
         assert 'This is an old revision of this dataset' in main_html
-        assert 'at Jan 02, 2011, 00:00' in main_html
+        assert 'at January 2, 2011, 00:00' in main_html
         self.check_named_element(main_html, 'a', 'href="/dataset/%s"' % self.pkg_name)
         print 'PKG', pkg_html
         assert 'title2' in res
@@ -567,7 +498,7 @@ class TestReadAtRevision(FunctionalTestCase, HtmlCheckMethods):
         print 'MAIN', main_html
         assert 'This is an old revision of this dataset' not in main_html
         assert 'This is the current revision of this dataset' in main_html
-        assert 'at Jan 03, 2011, 00:00' in main_html
+        assert 'at January 3, 2011, 00:00' in main_html
         self.check_named_element(main_html, 'a', 'href="/dataset/%s"' % self.pkg_name)
         print 'PKG', pkg_html
         assert 'title3' in res
@@ -930,8 +861,8 @@ class TestEdit(TestPackageForm):
     def test_edit_plugin_hook(self):
         # just the absolute basics
         try:
-            plugin = MockPackageControllerPlugin()
-            plugins.load(plugin)
+            plugins.load('test_package_controller_plugin')
+            plugin = plugins.get_plugin('test_package_controller_plugin')
             res = self.app.get(self.offset, extra_environ=self.extra_environ_admin)
             new_name = u'new-name'
             new_title = u'New Title'
@@ -942,15 +873,15 @@ class TestEdit(TestPackageForm):
             res = fv.submit('save', extra_environ=self.extra_environ_admin)
             # get redirected ...
             assert plugin.calls['edit'] == 1, plugin.calls
-            plugins.unload(plugin)
+            plugins.unload('test_package_controller_plugin')
         finally:
             self._reset_data()
 
     def test_after_update_plugin_hook(self):
         # just the absolute basics
         try:
-            plugin = MockPackageControllerPlugin()
-            plugins.load(plugin)
+            plugins.load('test_package_controller_plugin')
+            plugin = plugins.get_plugin('test_package_controller_plugin')
             res = self.app.get(self.offset, extra_environ=self.extra_environ_admin)
             new_name = u'new-name'
             new_title = u'New Title'
@@ -962,7 +893,7 @@ class TestEdit(TestPackageForm):
             # get redirected ...
             assert plugin.calls['after_update'] == 1, plugin.calls
             assert plugin.calls['after_create'] == 0, plugin.calls
-            plugins.unload(plugin)
+            plugins.unload('test_package_controller_plugin')
         finally:
             self._reset_data()
 
@@ -1020,15 +951,12 @@ class TestEdit(TestPackageForm):
         solr_url = SolrSettings.get()[0]
         try:
             SolrSettings.init(bad_solr_url)
-            plugins.load('synchronous_search')
 
             fv = self.res.forms['dataset-edit']
-            prefix = ''
             fv['log_message'] = u'Test log message'
             res = fv.submit('save', status=500, extra_environ=self.extra_environ_admin)
             assert 'Unable to update search index' in res, res
         finally:
-            plugins.unload('synchronous_search')
             SolrSettings.init(solr_url)
 
     def test_edit_pkg_with_relationships(self):
@@ -1081,22 +1009,22 @@ class TestDelete(TestPackageForm):
         model.repo.rebuild_db()
 
     def test_delete(self):
-        plugin = MockPackageControllerPlugin()
-        plugins.load(plugin)
+        plugins.load('test_package_controller_plugin')
+        plugin = plugins.get_plugin('test_package_controller_plugin')
 
         offset = url_for(controller='package', action='delete',
                 id='warandpeace')
-
-        self.app.post(offset, extra_environ=self.extra_environ_tester, status=401)
+        # Since organizations, any owned dataset can be edited/deleted by any
+        # user
+        self.app.post(offset, extra_environ=self.extra_environ_tester)
 
         self.app.post(offset, extra_environ=self.extra_environ_admin)
 
         assert model.Package.get('warandpeace').state == u'deleted'
 
-        assert plugin.calls['delete'] == 1
-        assert plugin.calls['after_delete'] == 1
-
-        plugins.unload(plugin)
+        assert plugin.calls['delete'] == 2
+        assert plugin.calls['after_delete'] == 2
+        plugins.unload('test_package_controller_plugin')
 
 
 class TestNew(TestPackageForm):
@@ -1292,8 +1220,8 @@ class TestNew(TestPackageForm):
         self._assert_form_errors(res)
 
     def test_new_plugin_hook(self):
-        plugin = MockPackageControllerPlugin()
-        plugins.load(plugin)
+        plugins.load('test_package_controller_plugin')
+        plugin = plugins.get_plugin('test_package_controller_plugin')
         offset = url_for(controller='package', action='new')
         res = self.app.get(offset, extra_environ=self.extra_environ_tester)
         new_name = u'plugged'
@@ -1304,11 +1232,11 @@ class TestNew(TestPackageForm):
         # get redirected ...
         assert plugin.calls['edit'] == 0, plugin.calls
         assert plugin.calls['create'] == 1, plugin.calls
-        plugins.unload(plugin)
+        plugins.unload('test_package_controller_plugin')
 
     def test_after_create_plugin_hook(self):
-        plugin = MockPackageControllerPlugin()
-        plugins.load(plugin)
+        plugins.load('test_package_controller_plugin')
+        plugin = plugins.get_plugin('test_package_controller_plugin')
         offset = url_for(controller='package', action='new')
         res = self.app.get(offset, extra_environ=self.extra_environ_tester)
         new_name = u'plugged2'
@@ -1321,15 +1249,13 @@ class TestNew(TestPackageForm):
         assert plugin.calls['after_create'] == 1, plugin.calls
 
         assert plugin.id_in_dict
-
-        plugins.unload(plugin)
+        plugins.unload('test_package_controller_plugin')
 
     def test_new_indexerror(self):
         bad_solr_url = 'http://127.0.0.1/badsolrurl'
         solr_url = SolrSettings.get()[0]
         try:
             SolrSettings.init(bad_solr_url)
-            plugins.load('synchronous_search')
             new_package_name = u'new-package-missing-solr'
 
             offset = url_for(controller='package', action='new')
@@ -1344,7 +1270,6 @@ class TestNew(TestPackageForm):
             res = fv.submit('save', status=500, extra_environ=self.extra_environ_tester)
             assert 'Unable to add package to search index' in res, res
         finally:
-            plugins.unload('synchronous_search')
             SolrSettings.init(solr_url)
 
     def test_change_locale(self):
@@ -1370,14 +1295,14 @@ class TestSearch(TestPackageForm):
         model.repo.rebuild_db()
 
     def test_search_plugin_hooks(self):
-        plugin = MockPackageControllerPlugin()
-        plugins.load(plugin)
+        plugins.load('test_package_controller_plugin')
+        plugin = plugins.get_plugin('test_package_controller_plugin')
         offset = url_for(controller='package', action='search')
         res = self.app.get(offset)
         # get redirected ...
         assert plugin.calls['before_search'] == 1, plugin.calls
         assert plugin.calls['after_search'] == 1, plugin.calls
-        plugins.unload(plugin)
+        plugins.unload('test_package_controller_plugin')
 
 class TestNewPreview(TestPackageBase):
     pkgname = u'testpkg'
@@ -1582,3 +1507,43 @@ class TestAutocomplete(PylonsTestCase, TestPackageBase):
         expected = ['A Wonderful Story (warandpeace)|warandpeace','annakarenina|annakarenina']
         received = sorted(res.body.split('\n'))
         assert expected == received
+
+class TestResourceListing(TestPackageBase):
+    @classmethod
+    def setup_class(cls):
+
+        CreateTestData.create()
+        cls.tester_user = model.User.by_name(u'tester')
+        cls.extra_environ_admin = {'REMOTE_USER': 'testsysadmin'}
+        cls.extra_environ_tester = {'REMOTE_USER': 'tester'}
+        cls.extra_environ_someone_else = {'REMOTE_USER': 'someone_else'}
+
+        tests.call_action_api(cls.app, 'organization_create',
+                                        name='test_org_2',
+                                        apikey=cls.tester_user.apikey)
+
+        tests.call_action_api(cls.app, 'package_create',
+                                        name='crimeandpunishment',
+                                        owner_org='test_org_2',
+                                        apikey=cls.tester_user.apikey)
+
+    @classmethod
+    def teardown_class(cls):
+        model.repo.rebuild_db()
+
+    def test_resource_listing_premissions_sysadmin(self):
+        # sysadmin 200
+         self.app.get('/dataset/resources/crimeandpunishment', extra_environ=self.extra_environ_admin, status=200)
+
+    def test_resource_listing_premissions_auth_user(self):
+        # auth user 200
+         self.app.get('/dataset/resources/crimeandpunishment', extra_environ=self.extra_environ_tester, status=200)
+
+    def test_resource_listing_premissions_non_auth_user(self):
+        # non auth user 401
+         self.app.get('/dataset/resources/crimeandpunishment', extra_environ=self.extra_environ_someone_else, status=[302,401])
+
+    def test_resource_listing_premissions_not_logged_in(self):
+        # not logged in 401
+         self.app.get('/dataset/resources/crimeandpunishment', status=[302,401])
+
