@@ -1,13 +1,12 @@
-import time
 from nose.tools import assert_equal, assert_raises
 from pylons import config
 from email.mime.text import MIMEText
 import hashlib
 
-from ckan import model
+import ckan.model as model
+import ckan.lib.mailer as mailer
 from ckan.tests.pylons_controller import PylonsTestCase
 from ckan.tests.mock_mail_server import SmtpServerHarness
-from ckan.lib.mailer import mail_recipient, mail_user, send_reset_link, add_msg_niceties, MailerException, get_reset_link_body, get_reset_link
 from ckan.lib.create_test_data import CreateTestData
 from ckan.lib.base import g
 
@@ -35,7 +34,7 @@ class TestMailer(SmtpServerHarness, PylonsTestCase):
     def mime_encode(self, msg, recipient_name):
         sender_name = g.site_title
         sender_url = g.site_url
-        body = add_msg_niceties(recipient_name, msg, sender_name, sender_url)
+        body = mailer.add_msg_niceties(recipient_name, msg, sender_name, sender_url)
         encoded_body = MIMEText(body.encode('utf-8'), 'plain', 'utf-8').get_payload().strip()
         return encoded_body
 
@@ -49,8 +48,7 @@ class TestMailer(SmtpServerHarness, PylonsTestCase):
                       'subject': 'Meeting', 
                       'body': 'The meeting is cancelled.',
                       'headers': {'header1': 'value1'}}
-        mail_recipient(**test_email)
-        time.sleep(0.1)
+        mailer.mail_recipient(**test_email)
 
         # check it went to the mock smtp server
         msgs = self.get_smtp_messages()
@@ -74,8 +72,7 @@ class TestMailer(SmtpServerHarness, PylonsTestCase):
                       'subject': 'Meeting', 
                       'body': 'The meeting is cancelled.',
                       'headers': {'header1': 'value1'}}
-        mail_user(**test_email)
-        time.sleep(0.1)
+        mailer.mail_user(**test_email)
 
         # check it went to the mock smtp server
         msgs = self.get_smtp_messages()
@@ -96,12 +93,11 @@ class TestMailer(SmtpServerHarness, PylonsTestCase):
                       'subject': 'Meeting', 
                       'body': 'The meeting is cancelled.',
                       'headers': {'header1': 'value1'}}
-        assert_raises(MailerException, mail_user, **test_email)
+        assert_raises(mailer.MailerException, mailer.mail_user, **test_email)
 
     def test_send_reset_email(self):
         # send email
-        send_reset_link(model.User.by_name(u'bob'))
-        time.sleep(0.1)
+        mailer.send_reset_link(model.User.by_name(u'bob'))
 
         # check it went to the mock smtp server
         msgs = self.get_smtp_messages()
@@ -110,9 +106,29 @@ class TestMailer(SmtpServerHarness, PylonsTestCase):
         assert_equal(msg[1], config['smtp.mail_from'])
         assert_equal(msg[2], [model.User.by_name(u'bob').email])
         assert 'Reset' in msg[3], msg[3]
-        test_msg = get_reset_link_body(model.User.by_name(u'bob'))
+        test_msg = mailer.get_reset_link_body(model.User.by_name(u'bob'))
+        expected_body = self.mime_encode(test_msg,
+                                         u'bob')
+        assert expected_body in msg[3], '%r not in %r' % (expected_body, msg[3])
+
+        # reset link tested in user functional test
+
+    def test_send_invite_email(self):
+        user = model.User.by_name(u'bob')
+        assert user.reset_key is None, user
+        # send email
+        mailer.send_invite(user)
+
+        # check it went to the mock smtp server
+        msgs = self.get_smtp_messages()
+        assert_equal(len(msgs), 1)
+        msg = msgs[0]
+        assert_equal(msg[1], config['smtp.mail_from'])
+        assert_equal(msg[2], [model.User.by_name(u'bob').email])
+        test_msg = mailer.get_invite_body(model.User.by_name(u'bob'))
         expected_body = self.mime_encode(test_msg,
                                          u'bob') 
         assert expected_body in msg[3], '%r not in %r' % (expected_body, msg[3])
+        assert user.reset_key is not None, user
         
         # reset link tested in user functional test
