@@ -5,7 +5,10 @@ from pprint import pprint
 from nose.tools import assert_equal, assert_raises
 from nose.plugins.skip import SkipTest
 from pylons import config
+import datetime
+import mock
 
+import vdm.sqlalchemy
 import ckan
 from ckan.lib.create_test_data import CreateTestData
 from ckan.lib.dictization.model_dictize import resource_dictize
@@ -59,8 +62,8 @@ class TestAction(WsgiAppCase):
         return json.loads(res.body)['result']
 
     def test_01_package_list(self):
-        postparams = '%s=1' % json.dumps({})
-        res = json.loads(self.app.post('/api/action/package_list', params=postparams).body)
+        res = json.loads(self.app.post('/api/action/package_list',
+                         headers={'content-type': 'application/json'}).body)
         assert res['success'] is True
         assert len(res['result']) == 2
         assert 'warandpeace' in res['result']
@@ -68,6 +71,13 @@ class TestAction(WsgiAppCase):
         assert res['help'].startswith(
             "Return a list of the names of the site's datasets (packages).")
 
+        postparams = '%s=1' % json.dumps({'limit': 1})
+        res = json.loads(self.app.post('/api/action/package_list',
+                         params=postparams).body)
+        assert res['success'] is True
+        assert len(res['result']) == 1
+        assert 'warandpeace' in res['result'] or 'annakarenina' in res['result']
+
 		# Test GET request
         res = json.loads(self.app.get('/api/action/package_list').body)
         assert len(res['result']) == 2
@@ -363,6 +373,11 @@ class TestAction(WsgiAppCase):
 
 
     def test_04_user_list(self):
+        # Create deleted user to make sure he won't appear in the user_list
+        deleted_user = CreateTestData.create_user('deleted_user')
+        deleted_user.delete()
+        model.repo.commit()
+
         postparams = '%s=1' % json.dumps({})
         res = self.app.post('/api/action/user_list', params=postparams)
         res_obj = json.loads(res.body)
@@ -400,7 +415,6 @@ class TestAction(WsgiAppCase):
         result = res_obj['result']
         assert result['name'] == 'annafan'
         assert 'apikey' in result
-        assert 'reset_key' in result
 
         # Sysadmin user can see everyone's api key
         res = self.app.post('/api/action/user_show', params=postparams,
@@ -410,7 +424,6 @@ class TestAction(WsgiAppCase):
         result = res_obj['result']
         assert result['name'] == 'annafan'
         assert 'apikey' in result
-        assert 'reset_key' in result
 
     def test_05_user_show_edits(self):
         postparams = '%s=1' % json.dumps({'id':'tester'})
@@ -545,10 +558,7 @@ class TestAction(WsgiAppCase):
 
         res_obj = json.loads(res.body)
         assert res_obj['help'].startswith("Update a user account.")
-        assert res_obj['error'] == {
-                '__type': 'Authorization Error',
-                'message': 'Access denied'
-            }
+        assert res_obj['error']['__type'] == 'Authorization Error'
         assert res_obj['success'] is False
 
     def test_12_user_update_errors(self):
@@ -584,6 +594,102 @@ class TestAction(WsgiAppCase):
             for expected_message in test_call['messages']:
                 assert expected_message[1] in ''.join(res_obj['error'][expected_message[0]])
 
+    @mock.patch('ckan.lib.mailer.send_invite')
+    def test_user_invite(self, send_invite):
+        email_username = 'invited_user$ckan'
+        email = '%s@email.com' % email_username
+        organization_name = 'an_organization'
+        CreateTestData.create_groups([{'name': organization_name}])
+        role = 'member'
+        organization = model.Group.get(organization_name)
+        params = {'email': email,
+                  'group_id': organization.id,
+                  'role': role}
+        postparams = '%s=1' % json.dumps(params)
+        extra_environ = {'Authorization': str(self.sysadmin_user.apikey)}
+
+        res = self.app.post('/api/action/user_invite', params=postparams,
+                            extra_environ=extra_environ)
+
+        res_obj = json.loads(res.body)
+        user = model.User.get(res_obj['result']['id'])
+        assert res_obj['success'] is True, res_obj
+        assert user.email == email, (user.email, email)
+        assert user.is_pending(), user
+        expected_username = email_username.replace('$', '-')
+        assert user.name.startswith(expected_username), (user.name,
+                                                         expected_username)
+        group_ids = user.get_group_ids(capacity=role)
+        assert organization.id in group_ids, (group_ids, organization.id)
+        assert send_invite.called
+        assert send_invite.call_args[0][0].id == res_obj['result']['id']
+
+    @mock.patch('ckan.lib.mailer.mail_user')
+    def test_user_invite_without_email_raises_error(self, mail_user):
+        user_dict = {}
+        postparams = '%s=1' % json.dumps(user_dict)
+        extra_environ = {'Authorization': str(self.sysadmin_user.apikey)}
+
+        res = self.app.post('/api/action/user_invite', params=postparams,
+                            extra_environ=extra_environ,
+                            status=StatusCodes.STATUS_409_CONFLICT)
+
+        res_obj = json.loads(res.body)
+        assert res_obj['success'] is False, res_obj
+        assert 'email' in res_obj['error'], res_obj
+
+    @mock.patch('random.SystemRandom')
+    def test_user_invite_should_work_even_if_tried_username_already_exists(self, system_random_mock):
+        patcher = mock.patch('ckan.lib.mailer.mail_user')
+        patcher.start()
+        email = 'invited_user@email.com'
+        organization_name = 'an_organization'
+        CreateTestData.create_groups([{'name': organization_name}])
+        role = 'member'
+        organization = model.Group.get(organization_name)
+        params = {'email': email,
+                  'group_id': organization.id,
+                  'role': role}
+        postparams = '%s=1' % json.dumps(params)
+        extra_environ = {'Authorization': str(self.sysadmin_user.apikey)}
+
+        system_random_mock.return_value.random.side_effect = [1000, 1000, 2000, 3000]
+
+        for _ in range(2):
+            res = self.app.post('/api/action/user_invite', params=postparams,
+                                extra_environ=extra_environ)
+
+            res_obj = json.loads(res.body)
+            assert res_obj['success'] is True, res_obj
+        patcher.stop()
+
+    def test_user_delete(self):
+        name = 'normal_user'
+        CreateTestData.create_user(name)
+        user = model.User.get(name)
+        user_dict = {'id': user.id}
+        postparams = '%s=1' % json.dumps(user_dict)
+
+        res = self.app.post('/api/action/user_delete', params=postparams,
+                            extra_environ={'Authorization': str(self.sysadmin_user.apikey)})
+
+        res_obj = json.loads(res.body)
+        deleted_user = model.User.get(name)
+        assert res_obj['success'] is True
+        assert deleted_user.is_deleted(), deleted_user
+
+    def test_user_delete_requires_data_dict_with_key_id(self):
+        user_dict = {'name': 'normal_user'}
+        postparams = '%s=1' % json.dumps(user_dict)
+
+        res = self.app.post('/api/action/user_delete', params=postparams,
+                            extra_environ={'Authorization': str(self.sysadmin_user.apikey)},
+                            status=StatusCodes.STATUS_409_CONFLICT)
+
+        res_obj = json.loads(res.body)
+        assert res_obj['success'] is False
+        assert res_obj['error']['id'] == ['Missing value']
+
     def test_13_group_list(self):
         postparams = '%s=1' % json.dumps({})
         res = self.app.post('/api/action/group_list', params=postparams)
@@ -662,6 +768,11 @@ class TestAction(WsgiAppCase):
         assert res_obj['success'] is False
 
     def test_16_user_autocomplete(self):
+        # Create deleted user to make sure he won't appear in the user_list
+        deleted_user = CreateTestData.create_user('joe')
+        deleted_user.delete()
+        model.repo.commit()
+
         #Empty query
         postparams = '%s=1' % json.dumps({})
         res = self.app.post(
@@ -721,10 +832,11 @@ class TestAction(WsgiAppCase):
 
         resource_updated.pop('url')
         resource_updated.pop('revision_id')
+        resource_updated.pop('revision_timestamp', None)
         resource_created.pop('url')
         resource_created.pop('revision_id')
-        resource_created.pop('revision_timestamp')
-        assert resource_updated == resource_created
+        resource_created.pop('revision_timestamp', None)
+        assert_equal(resource_updated, resource_created)
 
     def test_20_task_status_update(self):
         package_created = self._add_basic_package(u'test_task_status_update')
@@ -808,7 +920,7 @@ class TestAction(WsgiAppCase):
         res_obj = json.loads(res.body)
         assert res_obj['help'].startswith("Update a task status.")
         assert res_obj['success'] is False
-        assert res_obj['error'] == {'message': 'Access denied', '__type': 'Authorization Error'}
+        assert res_obj['error']['__type'] == 'Authorization Error'
 
     def test_23_task_status_validation(self):
         task_status = {}
@@ -898,8 +1010,13 @@ class TestAction(WsgiAppCase):
         postparams = '%s=1' % json.dumps({'id': resource.id})
         res = self.app.post('/api/action/resource_show', params=postparams)
         result = json.loads(res.body)['result']
+
+        # Remove tracking data from the result dict. This tracking data is
+        # added by the logic, so the other resource dict taken straight from
+        # resource_dictize() won't have it.
+        del result['tracking_summary']
+
         resource_dict = resource_dictize(resource, {'model': model})
-        result.pop('revision_timestamp')
         assert result == resource_dict, (result, resource_dict)
 
     def test_27_get_site_user_not_authorized(self):
@@ -1228,6 +1345,23 @@ class TestAction(WsgiAppCase):
         assert error['__type'] == 'Validation Error'
         assert error['extras_validation'] == ['Duplicate key "foo"']
 
+    def test_package_update_remove_org_error(self):
+        import ckan.tests
+        import paste.fixture
+        import pylons.test
+
+        app = paste.fixture.TestApp(pylons.test.pylonsapp)
+        org = ckan.tests.call_action_api(app, 'organization_create',
+                apikey=self.sysadmin_user.apikey, name='myorganization')
+        package = ckan.tests.call_action_api(app, 'package_create',
+                apikey=self.sysadmin_user.apikey, name='foobarbaz', owner_org=org['id'])
+
+        assert package['owner_org']
+        package['owner_org'] = ''
+        res = ckan.tests.call_action_api(app, 'package_update',
+                apikey=self.sysadmin_user.apikey, **package)
+        assert not res['owner_org'], res['owner_org']
+
     def test_package_update_duplicate_extras_error(self):
         import ckan.tests
         import paste.fixture
@@ -1382,6 +1516,48 @@ class TestActionPackageSearch(WsgiAppCase):
         assert_equal(result['count'], 1)
         assert_equal(result['results'][0]['name'], 'annakarenina')
 
+    def test_1_facet_limit(self):
+        params = {
+                'q':'*:*',
+                'facet.field': ['groups', 'tags', 'res_format', 'license'],
+                'rows': 20,
+                'start': 0,
+            }
+        postparams = '%s=1' % json.dumps(params)
+        res = self.app.post('/api/action/package_search', params=postparams)
+        res = json.loads(res.body)
+        assert_equal(res['success'], True)
+
+        assert_equal(len(res['result']['search_facets']['groups']['items']), 2)
+
+        params = {
+                'q':'*:*',
+                'facet.field': ['groups', 'tags', 'res_format', 'license'],
+                'facet.limit': 1,
+                'rows': 20,
+                'start': 0,
+            }
+        postparams = '%s=1' % json.dumps(params)
+        res = self.app.post('/api/action/package_search', params=postparams)
+        res = json.loads(res.body)
+        assert_equal(res['success'], True)
+
+        assert_equal(len(res['result']['search_facets']['groups']['items']), 1)
+
+        params = {
+                'q':'*:*',
+                'facet.field': ['groups', 'tags', 'res_format', 'license'],
+                'facet.limit': -1, # No limit
+                'rows': 20,
+                'start': 0,
+            }
+        postparams = '%s=1' % json.dumps(params)
+        res = self.app.post('/api/action/package_search', params=postparams)
+        res = json.loads(res.body)
+        assert_equal(res['success'], True)
+
+        assert_equal(len(res['result']['search_facets']['groups']['items']), 2)
+
     def test_1_basic_no_params(self):
         postparams = '%s=1' % json.dumps({})
         res = self.app.post('/api/action/package_search', params=postparams)
@@ -1431,6 +1607,8 @@ class TestActionPackageSearch(WsgiAppCase):
         rev = model.repo.new_revision()
         pkg = model.Package.get('warandpeace')
         pkg.title = "War and Peace [UPDATED]"
+
+        pkg.metadata_modified = datetime.datetime.utcnow()
         model.repo.commit_and_remove()
 
         res = self.app.post('/api/action/package_search', params=search_params)
@@ -1442,6 +1620,7 @@ class TestActionPackageSearch(WsgiAppCase):
         rev = model.repo.new_revision()
         pkg = model.Package.get('annakarenina')
         pkg.title = "A Novel By Tolstoy [UPDATED]"
+        pkg.metadata_modified = datetime.datetime.utcnow()
         model.repo.commit_and_remove()
 
         res = self.app.post('/api/action/package_search', params=search_params)
@@ -1670,3 +1849,225 @@ class TestBulkActions(WsgiAppCase):
 
         res = self.app.get('/api/action/package_search?q=*:*')
         assert json.loads(res.body)['result']['count'] == 0
+
+
+class TestGroupOrgView(WsgiAppCase):
+
+    @classmethod
+    def setup_class(cls):
+        model.Session.add_all([
+            model.User(name=u'sysadmin', apikey=u'sysadmin',
+                       password=u'sysadmin', sysadmin=True),
+        ])
+        model.Session.commit()
+
+        org_dict = '%s=1' % json.dumps({
+            'name': 'org',
+        })
+        res = cls.app.post('/api/action/organization_create',
+                            extra_environ={'Authorization': 'sysadmin'},
+                            params=org_dict)
+        cls.org_id = json.loads(res.body)['result']['id']
+
+        group_dict = '%s=1' % json.dumps({
+            'name': 'group',
+        })
+        res = cls.app.post('/api/action/group_create',
+                            extra_environ={'Authorization': 'sysadmin'},
+                            params=group_dict)
+        cls.group_id = json.loads(res.body)['result']['id']
+
+    @classmethod
+    def teardown_class(self):
+        model.repo.rebuild_db()
+
+    def test_1_view_org(self):
+        res = self.app.get('/api/action/organization_show',
+                params={'id': self.org_id})
+        res_json = json.loads(res.body)
+        assert res_json['success'] is True
+
+        res = self.app.get('/api/action/group_show',
+                params={'id': self.org_id}, expect_errors=True)
+        res_json = json.loads(res.body)
+        assert res_json['success'] is False
+
+    def test_2_view_group(self):
+        res = self.app.get('/api/action/group_show',
+                params={'id': self.group_id})
+        res_json = json.loads(res.body)
+        assert res_json['success'] is True
+
+        res = self.app.get('/api/action/organization_show',
+                params={'id': self.group_id}, expect_errors=True)
+        res_json = json.loads(res.body)
+        assert res_json['success'] is False
+
+
+class TestResourceAction(WsgiAppCase):
+
+    sysadmin_user = None
+
+    normal_user = None
+
+    @classmethod
+    def setup_class(cls):
+        search.clear()
+        CreateTestData.create()
+        cls.sysadmin_user = model.User.get('testsysadmin')
+
+    @classmethod
+    def teardown_class(cls):
+        model.repo.rebuild_db()
+
+    def _add_basic_package(self, package_name=u'test_package', **kwargs):
+        package = {
+            'name': package_name,
+            'title': u'A Novel By Tolstoy',
+            'resources': [{
+                'description': u'Full text.',
+                'format': u'plain text',
+                'url': u'http://www.annakarenina.com/download/'
+            }]
+        }
+        package.update(kwargs)
+
+        postparams = '%s=1' % json.dumps(package)
+        res = self.app.post('/api/action/package_create', params=postparams,
+                            extra_environ={'Authorization': 'tester'})
+        return json.loads(res.body)['result']
+
+    def test_01_delete_resource(self):
+        res_dict = self._add_basic_package()
+        pkg_id = res_dict['id']
+
+        resource_count = len(res_dict['resources'])
+        id = res_dict['resources'][0]['id']
+        url = '/api/action/resource_delete'
+
+        # Use the sysadmin user because this package doesn't belong to an org
+        res = self.app.post(url, params=json.dumps({'id': id}),
+                extra_environ={'Authorization': str(self.sysadmin_user.apikey)})
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is True
+
+        url = '/api/action/package_show'
+        res = self.app.get(url, {'id': pkg_id})
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is True
+        assert len(res_dict['result']['resources']) == resource_count - 1
+
+
+class TestMember(WsgiAppCase):
+
+    sysadmin = None
+
+    group = None
+
+    def setup(self):
+        username = 'sysadmin'
+        groupname = 'test group'
+        organization_name = 'test organization'
+        CreateTestData.create_user('sysadmin', **{ 'sysadmin': True })
+        CreateTestData.create_groups([{ 'name': groupname },
+                                      { 'name': organization_name,
+                                        'type': 'organization'}])
+        self.sysadmin = model.User.get(username)
+        self.group = model.Group.get(groupname)
+
+    def teardown(self):
+        model.repo.rebuild_db()
+
+    def test_group_member_create_works_user_id_and_group_id(self):
+        self._assert_we_can_add_user_to_group(self.sysadmin.id, self.group.id)
+
+    def test_group_member_create_works_with_user_id_and_group_name(self):
+        self._assert_we_can_add_user_to_group(self.sysadmin.id, self.group.name)
+
+    def test_group_member_create_works_with_user_name_and_group_name(self):
+        self._assert_we_can_add_user_to_group(self.sysadmin.name, self.group.name)
+
+    def _assert_we_can_add_user_to_group(self, user_id, group_id):
+        user = model.User.get(user_id)
+        group = model.Group.get(group_id)
+        url = '/api/action/group_member_create'
+        role = 'member'
+        postparams = '%s=1' % json.dumps({
+            'id': group_id,
+            'username': user_id,
+            'role': role})
+
+        res = self.app.post(url, params=postparams,
+                            extra_environ={'Authorization': str(user.apikey)})
+
+        res = json.loads(res.body)
+        groups = user.get_groups(group.type, role)
+        group_ids = [g.id for g in groups]
+        assert res['success'] is True, res
+        assert group.id in group_ids, (group, user_groups)
+
+
+class TestRelatedAction(WsgiAppCase):
+
+    sysadmin_user = None
+
+    normal_user = None
+
+    @classmethod
+    def setup_class(cls):
+        search.clear()
+        CreateTestData.create()
+        cls.sysadmin_user = model.User.get('testsysadmin')
+
+    @classmethod
+    def teardown_class(cls):
+        model.repo.rebuild_db()
+
+    def _add_basic_package(self, package_name=u'test_package', **kwargs):
+        package = {
+            'name': package_name,
+            'title': u'A Novel By Tolstoy',
+            'resources': [{
+                'description': u'Full text.',
+                'format': u'plain text',
+                'url': u'http://www.annakarenina.com/download/'
+            }]
+        }
+        package.update(kwargs)
+
+        postparams = '%s=1' % json.dumps(package)
+        res = self.app.post('/api/action/package_create', params=postparams,
+                            extra_environ={'Authorization': 'tester'})
+        return json.loads(res.body)['result']
+
+    def test_update_add_related_item(self):
+        package = self._add_basic_package()
+        related_item = {
+            "description": "Testing a Description",
+            "url": "http://example.com/image.png",
+            "title": "Testing",
+            "featured": 0,
+            "image_url": "http://example.com/image.png",
+            "type": "idea",
+            "dataset_id": package['id'],
+        }
+        related_item_json = json.dumps(related_item)
+        res_create = self.app.post('/api/action/related_create',
+                                   params=related_item_json,
+                                   extra_environ={'Authorization': 'tester'})
+        assert res_create.json['success']
+
+        related_update = res_create.json['result']
+        related_update = {'id': related_update['id'], 'title': 'Updated'}
+        related_update_json = json.dumps(related_update)
+        res_update = self.app.post('/api/action/related_update',
+                                   params=related_update_json,
+                                   extra_environ={'Authorization': 'tester'})
+        assert res_update.json['success']
+        res_update_json = res_update.json['result']
+        assert res_update_json['title'] == related_update['title']
+
+        related_item.pop('title')
+        related_item.pop('dataset_id')
+        for field in related_item:
+            assert related_item[field] == res_update_json[field]
