@@ -99,9 +99,7 @@ def cs_filter(f, filter_, ignore_comment_lines=True):
 
 class TestBadSpellings(object):
 
-    BAD_SPELLING_BLACKLIST_FILES = [
-        'ckan/lib/navl/__init__.py',
-    ]
+    BAD_SPELLING_BLACKLIST_FILES = []
 
     # these are the bad spellings with the correct spelling
     # use LOWER case
@@ -173,11 +171,8 @@ class TestNastyString(object):
     # The value is converted to a string anyway so the str() is unneeded in
     # any place.
 
-    NASTY_STR_BLACKLIST_FILES = [
-        'ckan/tests/functional/api/test_revision_search.py',
-        'ckan/tests/functional/test_pagination.py',
-        'ckan/tests/models/test_package_relationships.py',
-    ]
+    NASTY_STR_BLACKLIST_FILES = []
+
     fails = {}
     passes = []
     done = False
@@ -237,13 +232,10 @@ class TestImportFromCkan(object):
         'bin/ckan-hmg-breakdown.py',
         'bin/dump-ukgov.py',
         'ckan/config/middleware.py',
-        'ckan/config/routing.py',
         'ckan/controllers/error.py',
         'ckan/controllers/storage.py',
         'ckan/lib/authenticator.py',
-        'ckan/lib/base.py',
         'ckan/lib/munge.py',
-        'ckan/lib/package_saver.py',
         'ckan/lib/plugins.py',
         'ckan/lib/search/index.py',
         'ckan/lib/search/query.py',
@@ -259,7 +251,6 @@ class TestImportFromCkan(object):
         'ckan/migration/versions/035_harvesting_doc_versioning.py',
         'ckan/model/test_user.py',
         'ckan/plugins/__init__.py',
-        'ckan/plugins/core.py',
         'ckan/tests/__init__.py',
         'ckan/tests/ckantestplugin/ckantestplugin/__init__.py',
         'ckan/tests/functional/api/base.py',
@@ -341,7 +332,6 @@ class TestImportFromCkan(object):
         'ckanext/multilingual/plugin.py',
         'ckanext/reclinepreview/tests/test_preview.py',
         'ckanext/stats/controller.py',
-        'ckanext/stats/stats.py',
         'ckanext/stats/tests/__init__.py',
         'ckanext/stats/tests/test_stats_lib.py',
         'ckanext/stats/tests/test_stats_plugin.py',
@@ -400,9 +390,7 @@ class TestImportStar(object):
     # import * is bad for many reasons and should be avoided.
 
     IMPORT_STAR_BLACKLIST_FILES = [
-        'bin/ckan_spam.py',
         'ckan/lib/helpers.py',
-        'ckan/lib/package_saver.py',
         'ckan/migration/versions/001_add_existing_tables.py',
         'ckan/migration/versions/002_add_author_and_maintainer.py',
         'ckan/migration/versions/003_add_user_object.py',
@@ -484,7 +472,6 @@ class TestImportStar(object):
         'ckan/tests/lib/test_tag_search.py',
         'ckan/tests/misc/test_sync.py',
         'ckan/tests/models/test_extras.py',
-        'ckan/tests/models/test_group.py',
         'ckan/tests/models/test_misc.py',
         'ckan/tests/models/test_package.py',
         'ckan/tests/models/test_package_relationships.py',
@@ -495,7 +482,6 @@ class TestImportStar(object):
         'ckan/tests/pylons_controller.py',
         'ckan/tests/test_dumper.py',
         'ckan/tests/test_wsgi_ckanclient.py',
-        'ckanext/stats/stats.py',
         'fabfile.py',
     ]
     fails = {}
@@ -570,7 +556,6 @@ class TestPep8(object):
         'bin/webstore_test.py',
         'ckan/__init__.py',
         'ckan/ckan_nose_plugin.py',
-        'ckan/config/environment.py',
         'ckan/config/middleware.py',
         'ckan/config/routing.py',
         'ckan/config/sp_config.py',
@@ -588,7 +573,6 @@ class TestPep8(object):
         'ckan/lib/alphabet_paginate.py',
         'ckan/lib/app_globals.py',
         'ckan/lib/authenticator.py',
-        'ckan/lib/base.py',
         'ckan/lib/captcha.py',
         'ckan/lib/cli.py',
         'ckan/lib/create_test_data.py',
@@ -730,7 +714,6 @@ class TestPep8(object):
         'ckan/model/vocabulary.py',
         'ckan/new_authz.py',
         'ckan/pastertemplates/__init__.py',
-        'ckan/plugins/core.py',
         'ckan/plugins/interfaces.py',
         'ckan/plugins/toolkit.py',
         'ckan/poo.py',
@@ -767,7 +750,6 @@ class TestPep8(object):
         'ckan/tests/functional/test_cors.py',
         'ckan/tests/functional/test_error.py',
         'ckan/tests/functional/test_follow.py',
-        'ckan/tests/functional/test_group.py',
         'ckan/tests/functional/test_home.py',
         'ckan/tests/functional/test_package.py',
         'ckan/tests/functional/test_package_relationships.py',
@@ -844,7 +826,6 @@ class TestPep8(object):
         'ckanext/example_idatasetform/plugin.py',
         'ckanext/example_itemplatehelpers/plugin.py',
         'ckanext/multilingual/plugin.py',
-        'ckanext/multilingual/tests/test_multilingual_plugin.py',
         'ckanext/reclinepreview/plugin.py',
         'ckanext/reclinepreview/tests/test_preview.py',
         'ckanext/resourceproxy/plugin.py',
@@ -920,16 +901,13 @@ class TestActionAuth(object):
 
     ACTION_FN_SIGNATURES_BLACKLIST = [
         'create: activity_create',
-        'create: tag_create',
     ]
 
     ACTION_NO_AUTH_BLACKLIST = [
         'create: follow_dataset',
         'create: follow_group',
         'create: follow_user',
-        'create: member_create',
         'create: package_relationship_create_rest',
-        'delete: member_delete',
         'delete: package_relationship_delete_rest',
         'delete: unfollow_dataset',
         'delete: unfollow_group',
