@@ -1,7 +1,7 @@
 import ckan.plugins as p
 
 
-def _datastore_auth(context, data_dict, privilege='resource_update'):
+def datastore_auth(context, data_dict, privilege='resource_update'):
     if not 'id' in data_dict:
         data_dict['id'] = data_dict.get('resource_id')
     user = context.get('user')
@@ -19,20 +19,26 @@ def _datastore_auth(context, data_dict, privilege='resource_update'):
 
 
 def datastore_create(context, data_dict):
-    return _datastore_auth(context, data_dict)
+    return datastore_auth(context, data_dict)
 
 
 def datastore_upsert(context, data_dict):
-    return _datastore_auth(context, data_dict)
+    return datastore_auth(context, data_dict)
 
 
 def datastore_delete(context, data_dict):
-    return _datastore_auth(context, data_dict)
+    return datastore_auth(context, data_dict)
 
 
+@p.toolkit.auth_allow_anonymous_access
 def datastore_search(context, data_dict):
-    return _datastore_auth(context, data_dict, 'resource_show')
+    return datastore_auth(context, data_dict, 'resource_show')
+
+
+@p.toolkit.auth_allow_anonymous_access
+def datastore_search_sql(context, data_dict):
+    return {'success': True}
 
 
 def datastore_change_permissions(context, data_dict):
-    return _datastore_auth(context, data_dict)
+    return datastore_auth(context, data_dict)
