@@ -30,16 +30,19 @@ class TestDatastoreSearch(tests.WsgiAppCase):
         cls.resource = cls.dataset.resources[0]
         cls.data = {
             'resource_id': cls.resource.id,
+            'force': True,
             'aliases': 'books3',
             'fields': [{'id': u'b\xfck', 'type': 'text'},
                        {'id': 'author', 'type': 'text'},
                        {'id': 'published'},
-                       {'id': u'characters', u'type': u'_text'}],
+                       {'id': u'characters', u'type': u'_text'},
+                       {'id': 'rating with %'}],
             'records': [{u'b\xfck': 'annakarenina', 'author': 'tolstoy',
                         'published': '2005-03-01', 'nested': ['b', {'moo': 'moo'}],
-                        u'characters': [u'Princess Anna', u'Sergius']},
+                        u'characters': [u'Princess Anna', u'Sergius'],
+                        'rating with %': '60%'},
                         {u'b\xfck': 'warandpeace', 'author': 'tolstoy',
-                        'nested': {'a': 'b'}}
+                        'nested': {'a': 'b'}, 'rating with %': '99%'}
                        ]
         }
         postparams = '%s=1' % json.dumps(cls.data)
@@ -60,13 +63,15 @@ class TestDatastoreSearch(tests.WsgiAppCase):
                                  u'nested': [u'b', {u'moo': u'moo'}],
                                  u'b\xfck': u'annakarenina',
                                  u'author': u'tolstoy',
-                                 u'characters': [u'Princess Anna', u'Sergius']},
+                                 u'characters': [u'Princess Anna', u'Sergius'],
+                                 u'rating with %': u'60%'},
                                 {u'published': None,
                                  u'_id': 2,
                                  u'nested': {u'a': u'b'},
                                  u'b\xfck': u'warandpeace',
                                  u'author': u'tolstoy',
-                                 u'characters': None}]
+                                 u'characters': None,
+                                 u'rating with %': u'99%'}]
 
         engine = db._get_engine(
                 {'connection_url': pylons.config['ckan.datastore.write_url']}
@@ -112,7 +117,7 @@ class TestDatastoreSearch(tests.WsgiAppCase):
             context,
             {'name': 'privatedataset',
              'private': True,
-             'owner_org' : self.organization['id'],
+             'owner_org': self.organization['id'],
              'groups': [{
                  'id': group.id
              }]})
@@ -124,6 +129,7 @@ class TestDatastoreSearch(tests.WsgiAppCase):
 
         postparams = '%s=1' % json.dumps({
             'resource_id': resource['id'],
+            'force': True
         })
         auth = {'Authorization': str(self.sysadmin_user.apikey)}
         res = self.app.post('/api/action/datastore_create', params=postparams,
@@ -349,9 +355,10 @@ class TestDatastoreSearch(tests.WsgiAppCase):
         result = res_dict['result']
         assert result['total'] == 1
 
-        results = [extract(result['records'][0],
-            [u'_id', u'author', u'b\xfck', u'nested', u'published', u'characters'])]
-        assert results == [self.expected_records[0]], result['records']
+        results = [extract(result['records'][0], [
+            u'_id', u'author', u'b\xfck', u'nested',
+            u'published', u'characters', u'rating with %'])]
+        assert results == [self.expected_records[0]], results['records']
 
         data = {'resource_id': self.data['resource_id'],
                 'q': 'tolstoy'}
@@ -363,9 +370,10 @@ class TestDatastoreSearch(tests.WsgiAppCase):
         result = res_dict['result']
         assert result['total'] == 2
         results = [extract(
-                record,
-                [u'_id', u'author', u'b\xfck', u'nested', u'published', u'characters']
-            ) for record in result['records']]
+            record,
+            [u'_id', u'author', u'b\xfck', u'nested',
+             u'published', u'characters', u'rating with %']
+        ) for record in result['records']]
         assert results == self.expected_records, result['records']
 
         expected_fields = [{u'type': u'int4', u'id': u'_id'},
@@ -389,10 +397,10 @@ class TestDatastoreSearch(tests.WsgiAppCase):
         result = res_dict['result']
         assert result['total'] == 1
         results = [extract(
-                result['records'][0],
-                [u'_id', u'author', u'b\xfck', u'nested', u'published', u'characters']
-            )]
-        assert results == [self.expected_records[0]], result['records']
+            result['records'][0],
+            [u'_id', u'author', u'b\xfck', u'nested', u'published',
+             u'characters', u'rating with %'])]
+        assert results == [self.expected_records[0]], results['records']
 
         for field in expected_fields:
             assert field in result['fields'], field
@@ -419,6 +427,7 @@ class TestDatastoreFullTextSearch(tests.WsgiAppCase):
         resource = model.Package.get('annakarenina').resources[0]
         cls.data = dict(
             resource_id=resource.id,
+            force=True,
             fields=[
               {'id': 'id'},
               {'id': 'date', 'type':'date'},
@@ -482,8 +491,9 @@ class TestDatastoreSQL(tests.WsgiAppCase):
         if not tests.is_datastore_supported():
             raise nose.SkipTest("Datastore not supported")
         plugin = p.load('datastore')
-        plugin.configure(pylons.config)
         if plugin.legacy_mode:
+            # make sure we undo adding the plugin
+            p.unload('datastore')
             raise nose.SkipTest("SQL tests are not supported in legacy mode")
         ctd.CreateTestData.create()
         cls.sysadmin_user = model.User.get('testsysadmin')
@@ -492,6 +502,7 @@ class TestDatastoreSQL(tests.WsgiAppCase):
         resource = cls.dataset.resources[0]
         cls.data = {
             'resource_id': resource.id,
+            'force': True,
             'aliases': 'books4',
             'fields': [{'id': u'b\xfck', 'type': 'text'},
                        {'id': 'author', 'type': 'text'},
@@ -510,7 +521,7 @@ class TestDatastoreSQL(tests.WsgiAppCase):
                            extra_environ=auth)
         res_dict = json.loads(res.body)
         assert res_dict['success'] is True
-        
+
         # Make an organization, because private datasets must belong to one.
         cls.organization = tests.call_action_api(
             cls.app, 'organization_create',
@@ -662,6 +673,7 @@ class TestDatastoreSQL(tests.WsgiAppCase):
 
         postparams = '%s=1' % json.dumps({
             'resource_id': resource['id'],
+            'force': True
         })
         auth = {'Authorization': str(self.sysadmin_user.apikey)}
         res = self.app.post('/api/action/datastore_create', params=postparams,
@@ -701,7 +713,9 @@ class TestDatastoreSQL(tests.WsgiAppCase):
              'package_id': package['id']})
 
         postparams = '%s=1' % json.dumps({
-            'resource_id': resource['id']})
+            'resource_id': resource['id'],
+            'force': True
+        })
         auth = {'Authorization': str(self.sysadmin_user.apikey)}
         res = self.app.post('/api/action/datastore_create', params=postparams,
                             extra_environ=auth)
@@ -711,9 +725,9 @@ class TestDatastoreSQL(tests.WsgiAppCase):
         # Test public resource
         query = 'SELECT * FROM "{0}"'.format(resource['id'])
         data = {'sql': query}
-        postparams = json.dumps(data)
+        postparams_sql = json.dumps(data)
         auth = {'Authorization': str(self.normal_user.apikey)}
-        res = self.app.post('/api/action/datastore_search_sql', params=postparams,
+        res = self.app.post('/api/action/datastore_search_sql', params=postparams_sql,
                             extra_environ=auth)
         res_dict = json.loads(res.body)
         assert res_dict['success'] is True
@@ -725,7 +739,22 @@ class TestDatastoreSQL(tests.WsgiAppCase):
         package = p.toolkit.get_action('package_update')(context, package)
 
         # Test private
-        res = self.app.post('/api/action/datastore_search_sql', params=postparams,
+        res = self.app.post('/api/action/datastore_search_sql', params=postparams_sql,
+                            extra_environ=auth, status=403)
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is False
+        assert res_dict['error']['__type'] == 'Authorization Error'
+
+        postparams = json.dumps({'resource_id': resource['id']})
+        res = self.app.post('/api/action/datastore_search', params=postparams,
+                            extra_environ=auth, status=403)
+        res_dict = json.loads(res.body)
+        assert res_dict['success'] is False
+        assert res_dict['error']['__type'] == 'Authorization Error'
+
+        # we should not be able to make the private resource it public
+        postparams = json.dumps({'resource_id': resource['id']})
+        res = self.app.post('/api/action/datastore_make_public', params=postparams,
                             extra_environ=auth, status=403)
         res_dict = json.loads(res.body)
         assert res_dict['success'] is False
@@ -738,7 +767,7 @@ class TestDatastoreSQL(tests.WsgiAppCase):
         package = p.toolkit.get_action('package_update')(context, package)
 
         # Test public again
-        res = self.app.post('/api/action/datastore_search_sql', params=postparams,
+        res = self.app.post('/api/action/datastore_search_sql', params=postparams_sql,
                             extra_environ=auth)
         res_dict = json.loads(res.body)
         assert res_dict['success'] is True
