@@ -17,12 +17,12 @@ class TestPdfPreview(tests.WsgiAppCase):
 
     @classmethod
     def setup_class(cls):
-        cls._original_config = config.copy()
-        config['ckan.plugins'] = 'pdf_preview'
         wsgiapp = middleware.make_app(config['global_conf'], **config)
+        plugins.load('pdf_preview')
         cls.app = paste.fixture.TestApp(wsgiapp)
 
         cls.p = previewplugin.PdfPreview()
+        cls.p.proxy_is_enabled = False
 
         create_test_data.CreateTestData.create()
 
@@ -42,9 +42,7 @@ class TestPdfPreview(tests.WsgiAppCase):
 
     @classmethod
     def teardown_class(cls):
-        config.clear()
-        config.update(cls._original_config)
-        plugins.reset()
+        plugins.unload('pdf_preview')
         create_test_data.CreateTestData.delete()
 
     def test_can_preview(self):
